import { supabaseClient as supabase } from './supabase';
import { dbUser as dbUserStore, activeOrganization as activeOrganizationStore, currentUser as currentUserStore, } from '../stores/user';
import { notifications } from '../stores/notifications';
import { goto } from '$app/navigation';
const environment = import.meta.env.MODE;
let currentUser;
let dbUser;
let activeOrganization;
let created_by;
// const sendEmailVerification = false
dbUserStore.subscribe((value) => {
    if (value) {
        dbUser = value;
    }
});
currentUserStore.subscribe((value) => {
    if (value) {
        currentUser = value;
        created_by = value.id;
    }
});
activeOrganizationStore.subscribe((value) => {
    if (value) {
        activeOrganization = value;
    }
});
// *** Common ***/
function handleError(error) {
    console.error(error);
    notifications.error(error.message);
}
function crud(table) {
    return {
        async get(id) {
            return await getRecord(table, id);
        },
        async getMany(match = {}) {
            return await getRecords(table, match);
        },
        async create(payload, message) {
            return await createRecord(table, payload, message);
        },
        async update(id, data, message) {
            return await updateRecord(table, id, data, message);
        },
        async delete(id, message) {
            return await deleteRecord(table, id, message);
        },
    };
}
// const Foo = {
// 	...crud<DbUser>('users'),
// }
export const db = {
    users: crud('users'),
    organizations: crud('organizations'),
    webhooks: crud('webhooks'),
};
// export const Users = {
// 	...crud<DbUser>('users')
// }
// db.users.get(1).then(res => res.)
// Foo.get(1).then((res) => res)
// Foo.getMany({ display_name: 'foo' }).then((res) =>
// 	res.forEach((r) => r.description)
// )
// Foo.update(1, { displayName: 'foo' })
// Foo.create({ display_name: 'foo' })
// Foo.delete(1)
// https://supabase.com/docs/reference/javascript/update
async function handleQuery(supabaseQuery, message) {
    const { data, error } = await supabaseQuery;
    if (error) {
        handleError(error);
    }
    else if (message) {
        notifications.success(message);
    }
    return data || error;
}
async function getRecord(from, id, select = '*') {
    const { data, error } = await supabase
        .from(from)
        .select(select)
        .eq('id', id)
        .single();
    if (error)
        handleError(error);
    return data;
}
async function getRecords(from, match = {}) {
    const { data, error } = await supabase.from(from).select().match(match);
    if (error)
        handleError(error);
    return data?.length ? data : [];
}
async function createRecord(table, payload, message) {
    const { data, error } = await supabase
        .from(table)
        .insert(payload)
        .select()
        .single();
    if (error) {
        return handleError(error);
        return null;
    }
    else if (message) {
        notifications.success(message);
    }
    return data;
    // return body?.[0] || error
}
async function deleteRecord(table, id, message) {
    const { data, error } = await supabase.from(table).delete().match({ id });
    if (error) {
        return handleError(error);
    }
    else if (message) {
        notifications.success(message);
    }
    return data;
}
export async function upsertRecord(table, body, message) {
    const { data, error } = await supabase
        .from(table)
        .upsert(body)
        .select()
        .maybeSingle();
    if (error) {
        return handleError(error);
    }
    else if (message) {
        notifications.success(message);
    }
    return data;
}
export async function updateRecord(table, id, data, message) {
    return await handleQuery(supabase.from(table).update(data).eq('id', id).select().maybeSingle(), message);
}
// *** End Common ***/
export const Admin = {
    users: {
        getMany(limit, offset) {
            return handleQuery(supabase
                .from('users')
                .select('*')
                .range(offset, offset + limit));
        },
        search(email) {
            return handleQuery(supabase.from('users').select('*').eq('email', email));
        },
    },
};
async function uploadFile(file, bucket, path) {
    const { error } = await supabase.storage
        .from(bucket)
        .upload(path, file, { upsert: true });
    if (error) {
        handleError(error);
        return null;
    }
    else {
        // const url = await supabase.storage.from('avatars').download(path)
        const { data } = await supabase.storage.from(bucket).getPublicUrl(path);
        return data.publicUrl;
    }
}
export const Auth = {
    get user() {
        return currentUser;
    },
    async resetPassword({ email, continueUrl, }) {
        let { data, error } = await supabase.auth.resetPasswordForEmail(email, {
            redirectTo: continueUrl,
        });
        console.log(data, error);
        if (error) {
            handleError(error);
        }
        else {
            notifications.success('Check your email for a password reset link');
        }
    },
    async sendPasswordResetEmail(email) {
        const { data, error } = await supabase.auth.resetPasswordForEmail(email);
        if (error)
            handleError(error);
        return data;
    },
    async signIn({ email, password }) {
        const { data, error } = await supabase.auth.signInWithPassword({
            email,
            password,
        });
        console.log(data);
        if (error)
            handleError(error);
    },
    async signInWithProvider(provider = 'github') {
        const { data, error } = await supabase.auth.signInWithOAuth({
            provider,
            // options: {
            // 	redirectTo: `${window.location.origin}/auth/callback`,
            // 	scopes: 'read:user',
            // }
        });
        if (error)
            handleError(error);
        return data;
    },
    signInWithoutPassword({ email }) {
        if (environment === 'development') {
            return supabase.auth.signInWithOtp({
                email,
                options: { emailRedirectTo: 'http://localhost:3000/' },
            });
        }
        return supabase.auth.signInWithOtp({ email });
    },
    signOut() {
        return supabase.auth.signOut();
    },
    async signUp({ email, password, metadata = {}, }) {
        let { error } = await supabase.auth.signUp({
            email,
            password,
            options: { data: metadata },
        });
        if (error) {
            handleError(error);
        }
        else {
            goto('/');
        }
    },
    update(data) {
        return supabase.auth.updateUser(data);
    },
};
export const Users = {
    table: 'users',
    ...crud('users'),
    async addPhoto(file) {
        const path = `users/${currentUser.id}`;
        return await uploadFile(file, 'avatars', path);
    },
};
export const Organizations = {
    table: 'organizations',
    get organization() {
        return activeOrganization;
    },
    ...crud('organizations'),
    async create(body) {
        return createRecord(this.table, {
            ...body,
            created_by,
        }, 'Success');
    },
    async getForUser(userId) {
        if (!userId)
            userId = currentUser.id;
        const { data, error } = await supabase
            .from('user_organization')
            .select('*, organization:organizations(*)')
            .eq('user_id', userId)
            .order('last_active', { ascending: false });
        if (error)
            handleError(error);
        if (!data)
            return [];
        const mapped = data.map((val) => {
            const o = val.organization;
            return {
                role: val.role,
                id: o.id,
                name: o.name,
                description: o.description,
                website: o.website,
                photo_url: o.photo_url,
                created_at: o.created_at,
                updated_at: o.updated_at,
                created_by: o.created_by,
            };
        });
        return mapped || [];
    },
    async addPhoto(file) {
        const path = `organizations/${activeOrganization.id}`;
        return await uploadFile(file, 'avatars', path);
    },
    async update(id, body) {
        const { data, error } = await supabase
            .from(this.table)
            .update(body)
            .match({ id: id || activeOrganization.id });
        if (error)
            handleError(error);
        return data || {};
    },
    users: {
        add: async (user_id, organization_id, role) => {
            const { data, error } = await supabase.from('user_organization').insert({
                user_id,
                organization_id: activeOrganization.id,
                role,
                created_by,
            });
            if (error)
                handleError(error);
            return data;
        },
        remove: async (user_id) => {
            const { data, error } = await supabase
                .from('user_organization')
                .delete()
                .match({ user_id, organization_id: activeOrganization.id });
            if (error)
                handleError(error);
            return data;
        },
        async get() {
            if (!activeOrganization?.id)
                return console.log('yo', currentUser, dbUser);
            // const { data, error } = await supabase
            // 	.from('user_organization')
            // 	.select('*, users!user_organization_user_id_fkey(*)')
            // 	.eq('organization_id', activeOrganization.id)
            // const { data, error } = await supabase
            // 	.from('user_organization')
            // 	.select('*, users:user_id(*):org_role')
            // 	.eq('organization_id', activeOrganization.id)
            const { data, error } = await supabase
                .from('users')
                .select('*, user_organization!user_organization_user_id_fkey!inner(*)')
                .eq('user_organization.organization_id', activeOrganization.id);
            // .order('last_active', { ascending: false })
            if (error)
                handleError(error);
            console.log(data);
            const mapped = data?.map((i) => {
                return {
                    ...i,
                    role: i.user_organization[0].role,
                };
            });
            return mapped || [];
        },
        async getOne(user_id, organization_id) {
            return handleQuery(supabase
                .from('user_organization')
                .select()
                .eq('user_id', user_id)
                .eq('organization_id', organization_id)
                .single());
        },
        async update(user_id, body) {
            const { data, error } = await supabase
                .from('user_organization')
                .update(body)
                .match({ organization_id: activeOrganization.id, user_id });
            data?.forEach((r) => r);
            if (error)
                handleError(error);
            notifications.success('Updated');
        },
    },
};
export const Files = {
    async get() {
        let { data } = await supabase.storage
            .from('avatars')
            .getPublicUrl(activeOrganization.id);
        console.log(data);
        return data.publicUrl;
    },
};
export const Notifications = {
    get() {
        return [];
    },
    updateMany(notifications = [], data) {
        let ids = notifications.map((d) => d.id);
        return handleQuery(supabase.from('user_notifications').update(data).in('id', ids));
    },
    updatePreferences(data) { },
};
export const Projects = {
    ...crud('projects'),
};
export const Messages = {
    async create(threadId, data) {
        await Promise.all([
            handleQuery(supabase
                .from('messages')
                .insert({ ...data, thread_id: threadId, created_by })),
            handleQuery(supabase
                .from('message_threads')
                .update({
                last_message_by: created_by,
                last_message_at: new Date(),
                last_message: data.text,
            })
                .eq('id', threadId)),
        ]);
        notifications.success('Message sent');
    },
    async update(id) {
        return updateRecord('messages', id, { archived: true });
    },
    async getThreads() {
        return await handleQuery(supabase
            .from('message_threads')
            .select('*,preview:users!message_threads_last_message_by_fkey(display_name,photo_url)')
            .order('last_message_at', { ascending: false }));
    },
    async getForThread(threadId) {
        return await handleQuery(supabase
            .from('messages')
            .select('*,preview:messages_created_by_fkey(display_name,photo_url)')
            .eq('thread_id', threadId));
    },
};
export const Webhooks = {
    ...crud('webhooks'),
};
